/*
{
  "DESCRIPTION": "http://glslsandbox.com/e#54154.2",
  "INPUTS": [
 	{
      "NAME": "Bright",
      "TYPE": "float",
      "DEFAULT": 0.00,
      "MIN": 0.00,
      "MAX": 0.90
    },
	{
      "NAME": "Hue",
      "TYPE": "float",
      "DEFAULT": 2.5,
      "MIN": -100.0,
      "MAX": 100.0
    },
  	{
      "NAME": "Exposure",
      "TYPE": "float",
      "DEFAULT": 0.00,
      "MIN": -10.0,
      "MAX": 1.0
    }, 	
	{
      "NAME": "BandSpeed",
      "TYPE": "float",
      "DEFAULT": 20,
      "MIN": 0,
      "MAX": 100
    },
	{
      "NAME": "BandCount",
      "TYPE": "float",
      "DEFAULT": 30.0,
      "MIN": 0.0,
      "MAX": 100.0
    },	
	{
      "NAME": "Direction",
      "TYPE": "float",
      "DEFAULT": 0.5,
      "MIN": -1.0,
      "MAX": 1.0
    },	
	{
      "NAME": "SpinSpeed",
      "TYPE": "float",
      "DEFAULT": 0.5,
      "MIN": 0.0,
      "MAX": 2.0
    },
	{
      "NAME": "Wobble",
      "TYPE": "float",
      "DEFAULT": 1.0,
      "MIN": 1.0,
      "MAX": 5.0
    },
	{
      "NAME": "Wobble2",
      "TYPE": "float",
      "DEFAULT": 0.0,
      "MIN": -5.0,
      "MAX": 5.0
    },
	{
      "NAME": "AbstractOffset",
      "TYPE": "float",
      "DEFAULT": 0.0,
      "MIN": -1.9,
      "MAX": 1.9
    },
	{
      "NAME": "WaveCount",
      "TYPE": "float",
      "DEFAULT": 6.0,
      "MIN": 1.0,
      "MAX": 100.0
    },
	{
      "NAME": "Skinny",
      "TYPE": "float",
      "DEFAULT": 10.0,
      "MIN": 1.0,
      "MAX": 100.0
    },
	{
      "NAME": "Height",
      "TYPE": "float",
      "DEFAULT": 0.65,
      "MIN": 0.01,
      "MAX": 0.99
    },	
	{
      "NAME": "RibbonBright",
      "TYPE": "float",
      "DEFAULT": 1.0,
      "MIN": 0.1,
      "MAX": 2.0
    },
	{
      "NAME": "ZoomView",
      "TYPE": "float",
      "DEFAULT": 0.2,
      "MIN": 0.0,
      "MAX": 1.0
    },
	{
      "NAME": "AngleView",
      "TYPE": "float",
      "DEFAULT": 0.5,
      "MIN": 0.01,
      "MAX": 0.99
    },
	{
      "NAME": "ZoomBounce",
      "TYPE": "float",
      "DEFAULT": 5.0,
      "MIN": 0.0,
      "MAX": 10.0
    },
	{
      "NAME": "Invert",
      "TYPE": "float",
      "DEFAULT": 0.1,
      "MIN": -0.1,
      "MAX": 0.1
    },
	{
      "NAME": "Rotate",
      "TYPE": "float",
      "DEFAULT": 0.0,
      "MIN": -1.9,
      "MAX": 1.9
    },

	{
			"NAME": "C2",
			"TYPE": "color",
			"DEFAULT" :   [ 0.5,0.75,0.75 1.0 ]
		},
		{
			"NAME": "C3",
			"TYPE": "color",
			"DEFAULT" :   [ 0.9, 0.0, 0.9, 1.0 ]
		} 
  ]
*/


#ifdef GL_ES
precision mediump float;
#endif



void main( void ) {

	vec2 uv =  (gl_FragCoord.xy -AngleView * RENDERSIZE.xy) / RENDERSIZE.y ;
	
	float t = TIME * SpinSpeed;
	
	vec3 ro = vec3(0, 0, -1);
    	vec3 lookat = vec3(Wobble2);
    	float zoom = ZoomView + abs( sin(t))/ZoomBounce;
    
    	vec3 f = normalize(lookat-ro),
        r = normalize(cross(vec3(Rotate,Invert,0), f)),
        u = cross(f, r),
        c = ro + f * zoom,
        i = c + uv.x * r + uv.y * u,
        rd = normalize(i-ro);
    
    	float radius = mix(.3, 1.5, .5+.5);
    
    	float dS, dO = AbstractOffset;
    	vec3 p;
    
    	for(int i=0; i<1000; i++) {
    		p = ro + rd * dO;
        	dS = -(length(vec2(length(p.xz)-1., p.y)) - Height);
        	if(dS<.0001) break;
        	dO += dS;
    }
    
    vec3 col = vec3(Exposure);

    if(dS<.001) {
    	float x = atan(p.x, p.z)+t*Direction;
        float y = atan(length(p.xz)-Wobble, p.y);
        
        float bands = sin(y*Skinny+x*BandCount);
        float ripples = sin((x*10.-y*30.)*3.)*.5+.5;
        float waves = sin(x*2.-y*WaveCount+t*BandSpeed);
        
        float b1 = smoothstep(-.2, .2, bands);
        float b2 = smoothstep(-.2, .2, bands-.5);
        
        float m = b1*(RibbonBright-b2);
        m = max(m, ripples*b2*max(0., waves));
        m += max(Bright, waves*.3*b2);
		
	float fd = length(ro-p);
        col += m;
	col.rb *= Hue;
	col.z *= 2.5*abs(cos(t));
	col = mix(C3.xyz,C2.xyz, m);
    }
	
	gl_FragColor = vec4( col, 1.0 );
}